<?php
define('WPWHOOSH_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('WPWHOOSH_ADMIN', WPWHOOSH_PLUGIN_NAME.'_admin');
define('WPWHOOSH_KEY', 'wpwhoosh_key');
define('WPWHOOSH_TEMPLATES', 'wpwhoosh_templates');
define('WPWHOOSH_CREDITS', 'wpwhoosh_credits');
define('WPWHOOSH_HOSTS', 'wpwhoosh_hosts');
define('WPWHOOSH_SITES', 'wpwhoosh_sites');

require_once (WPWHOOSH_PLUGIN_DIR.'functions.php');
require_once (WPWHOOSH_PLUGIN_DIR.'updater.php');
require_once (WPWHOOSH_PLUGIN_DIR.'list.php');
require_once (WPWHOOSH_PLUGIN_DIR.'key.php');
require_once (WPWHOOSH_PLUGIN_DIR.'template-factory.php');
require_once (WPWHOOSH_PLUGIN_DIR.'templates.php');
require_once (WPWHOOSH_PLUGIN_DIR.'credits.php');
require_once (WPWHOOSH_PLUGIN_DIR.'hosts.php');
require_once (WPWHOOSH_PLUGIN_DIR.'sites.php');

$wpwhoosh_admin = new wpwhoosh_admin();

class wpwhoosh_admin {

    private $pagehook;

	function __construct() {
		add_filter('screen_layout_columns', array(&$this, 'screen_layout_columns'), 10, 2);
		add_filter('plugin_action_links',array(&$this, 'plugin_action_links'), 10, 2 );
		add_action('admin_menu', array(&$this,'admin_menu'));
		add_action('init', array(WPWHOOSH_KEY,'init'));
		add_action('init', array(WPWHOOSH_TEMPLATES,'init'));
		add_action('init', array(WPWHOOSH_CREDITS,'init'));
		add_action('init', array(WPWHOOSH_HOSTS,'init'));
		add_action('init', array(WPWHOOSH_SITES,'init'));
	}

	function screen_layout_columns($columns, $screen) {
		if (!defined( 'WP_NETWORK_ADMIN' ) && !defined( 'WP_USER_ADMIN' )) {
			if ($screen == $this->pagehook) {
				$columns[$this->pagehook] = 2;
			}
		}
		return $columns;
	}

	function plugin_action_links( $links, $file ) {
		if ( is_array($links) && (WPWHOOSH_PATH == $file )) {
			$settings_link = '<a href="' . admin_url( 'admin.php?page='.WPWHOOSH_PLUGIN_NAME) . '">Settings</a>';
			array_unshift( $links, $settings_link );
		}
		return $links;
	}

	function admin_menu() {
		$this->pagehook = add_menu_page(WPWHOOSH_FRIENDLY_NAME, WPWHOOSH_FRIENDLY_NAME, 'read', 
			WPWHOOSH_PLUGIN_NAME, array(&$this,'resources_panel'),WPWHOOSH_IMAGES_URL.'menu-icon.png' );
	}
	
	function resources_panel() {
    	$templates_url = WPWHOOSH_TEMPLATES::get_url(); 
    	$key_url = WPWHOOSH_KEY::get_url(); 
    	$hosts_url = WPWHOOSH_HOSTS::get_url(); 
    	$sites_url = WPWHOOSH_SITES::get_url(); 
    	$credits_url = WPWHOOSH_CREDITS::get_url(); 
    	$home_url = WPWHOOSH_HOME_URL;
    	$version = WPWHOOSH_VERSION;
    	$plugin = WPWHOOSH_FRIENDLY_NAME;
    	print <<< ADMIN_PANEL
<div class="wrap">
<h2>{$plugin} {$version} Instructions</h2>
<p>{$plugin} allows you to make a professional and secure installation of WordPress with good performance and SEO in under 60 seconds.</p>
<p>A {$plugin} installation uses a customized template. The template comprises a chosen WordPress theme, our selection of quality plugins, standard pages 
(contact, terms and conditions, privacy statement), and example posts, or real posts if your chose one of our niche specific templates. 
The installation also includes items such as .htaccess, robots.txt, sitemap.xml in order to improve the security and performance of the site.</p> 
<p>Using {$plugin} will not only give you peace of mind with a more secure installation of WordPress but will save you precious time: typically something 
between 15 minutes to 120 minutes in getting your WordPress site up and running.</p> 
<h3>Getting Started</h3>
<p>Start by getting your <a href="{$key_url}">API Key</a></p>
<p>Once you have saved your key you will be able to see what <a href="{$templates_url}">{$plugin} Templates</a> are 
available for installation</a>.</p>
<h3>Using the Plugin To Install WordPress on New Sites</h3>
<p>Once you have <a href="{$hosts_url}">set up a host</a> and purchased some <a href="{$credits_url}">credits</a> go to <a href="{$sites_url}">sites</a> to install WordPress securely, easily and professionally.</p>
<h3>Getting Support</h3>
<p>For plugin features and capabilities go to <a href="{$home_url}/features/">Features</a></p>
<p>For frequently asked questions go to <a href="{$home_url}/faq/">FAQ</a></p>
<p>To learn how to install WordPress professionally see <a href="{$home_url}/tutorials/">Tutorials</a></p>
<p>For a 60 second installation of WordPress see <a href="{$home_url}/demo/">Demonstration</a></p>
<p>For help go to <a href="{$home_url}/help/">Help</a></p>
</div>
ADMIN_PANEL;
	}
}
?>