<?php
class wpwhoosh_hosts {

    private static $initialized = false;
    private static $slug = 'wpwhoosh_hosts';
    private static $parenthook = WPWHOOSH_PLUGIN_NAME;
    private static $screen_id;
    private static $list;    
    private static $keys = array('host_id', 'host_name', 'host_ftp', 'cpanel_provider', 'cpanel_url', 'cpanel_user', 'cpanel_password', 'cpanel_status', 'cpanel_verified_date' );
	private static $tips = array(
			'host_name' => array('heading' => 'Host Name', 'tip' => 'Supply a short unique name for this hosting account - this is useful if you have several accounts with the same host: eg My HostGator Baby.'),
			'cpanel_provider' => array ('heading' => 'Host Provider', 'tip' => 'Choose your host. This will help us to choose default values for some fields.'),
			'cpanel_url' => array('heading' => 'cPanel URL', 'tip' => 'The full URL of your cPanel e.g https://gator123.hostgator.com:2083.'),
			'cpanel_user' => array('heading' => 'cPanel User', 'tip' => 'Your CPanel Username. This is normally 8 characters in length.'), 
			'cpanel_password' => array('heading' => 'cPanel Password', 'tip' => 'Your CPanel Password. Your password is always transferred and stored in an encrypted format for reasons of security.'), 
			'cpanel_status' => array('heading' => 'Host Status', 'tip' => 'If a host is unverified it means either the cPanel URL, username or password is incorrect and hence it is not possible to deploy a site to that host.') 
	);
	private static $tooltips;

	static function init() {
	    if (self::$initialized) return true;
		self::$initialized = true;
		add_action('admin_menu',array(WPWHOOSH_HOSTS, 'admin_menu'));
		self::$tooltips = new wpwhoosh_tooltip(self::$tips);
	}

	static function admin_menu() {
		self::$screen_id = add_submenu_page(self::get_parenthook(), __('Host Boss List Hosts',WPWHOOSH_PLUGIN_NAME), 
			__('Hosts',WPWHOOSH_PLUGIN_NAME), 'read', self::get_slug(), array(WPWHOOSH_HOSTS,'controller'));
		add_filter('screen_layout_columns', array(WPWHOOSH_HOSTS, 'screen_layout_columns'), 10, 2);
		add_action('load-'.self::get_screen_id(), array(WPWHOOSH_HOSTS, 'load_page'));
		add_action('admin_head-'.self::get_screen_id(), array(WPWHOOSH_HOSTS, 'load_style'));
		add_action('admin_footer-'.self::get_screen_id(), array(WPWHOOSH_HOSTS, 'load_script'));		
		add_action('admin_footer-'.self::get_screen_id(), array(WPWHOOSH_HOSTS, 'toggle_postboxes'));
	}

    static function get_slug() {
		return self::$slug;
	}

    static function get_parenthook(){
		return self::$parenthook;
	}

    static function get_screen_id(){
		return self::$screen_id;
	}

    static function get_list() {
		return self::$list;
	}

	static function set_list($list) {
		self::$list = $list;
	}	

    static function get_keys() {
		return self::$keys;
	}
	
 	public static function get_url($id='', $noheader = false) {
		return WPWhooshUtils::admin_url(self::get_slug(), '', $id, $noheader);
	}

	static function enable_screen($show_screen,$screen) {
		if ($screen->id == self::get_screen_id())
			return true;
		else
			return $show_screen;
	}	
	
	static function screen_layout_columns($columns, $screen) {
		if (!defined( 'WP_NETWORK_ADMIN' ) && !defined( 'WP_USER_ADMIN' )) {
			if ($screen == self::get_screen_id()) {
				$columns[self::get_screen_id()] = 1;
			}
		}
		return $columns;
	}

	static function load_style() {
    	printf ('<link rel="stylesheet" id="%1$s" href="%2$s?ver=%3$s" type="text/css" media="all" />',
			WPWHOOSH_HOSTS, WPWHOOSH_PLUGIN_URL.'hosts.css', WPWHOOSH_VERSION);	
 	}

	static function load_script() {
		printf ('<script type="text/javascript" src="%1$s?ver=%2$s"></script>',
			'https://ajax.aspnetcdn.com/ajax/jquery.validate/1.9/jquery.validate.min.js','1.9');
		printf ('<script type="text/javascript" src="%1$s?ver=%2$s"></script>',
			'https://ajax.aspnetcdn.com/ajax/jquery.validate/1.9/additional-methods.js','1.9');
		printf ('<script type="text/javascript" src="%1$s?ver=%2$s"></script>',
    		WPWHOOSH_PLUGIN_URL.'hosts.js', WPWHOOSH_VERSION);   	
	}	

	static function load_page() {
		add_filter('screen_options_show_screen', array(WPWHOOSH_HOSTS,'enable_screen'),10,2);
		add_filter('screen_layout_columns', array(WPWHOOSH_HOSTS, 'screen_layout_columns'), 10, 2);	
		wp_enqueue_script('common');
		wp_enqueue_script('wp-lists');
		wp_enqueue_script('postbox');
		$id = array_key_exists('id', $_GET) ? $_GET['id'] : 0;
		$action =  array_key_exists('action',$_REQUEST) ? $_REQUEST['action'] : -1;		
		if (! empty($id) ||  ($action=='add')  || ($action=='save')) 
			self::load_page_edit($id) ;
		else 
	    	self::load_page_list();	
    }

	static function load_page_list() {
		require_once (WPWHOOSH_PLUGIN_DIR.'host-list-table.php');
		self::set_list(new wpwhoosh_host_list_table(self::get_url()));
		add_filter ('manage_' . self::get_screen_id() . '_columns', array(self::get_list(),'get_columns'));			
		$current_screen = get_current_screen();
		if (method_exists($current_screen,'add_help_tab')) {		
			$current_screen->add_help_tab( array( 'id' => 'wpwhoosh_hosts_overview', 'title' => 'Overview', 		
			'content' => sprintf('<h3>%1$s</h3><p>%2$s. %3$s</p><p><a rel="external" href="%4$s">%5$s %1$s</a></p>',
				WPWHOOSH_FRIENDLY_NAME, 
				__('Here you can update your host details',WPWHOOSH_PLUGIN_NAME),
				__('Click the link below for detailed instructions.',WPWHOOSH_PLUGIN_NAME),
				WPWHOOSH_HOME_URL.'/tutorials/',
				__('Getting Started with',WPWHOOSH_PLUGIN_NAME)
			)));		
		}
	}

	static function load_page_edit($host_id) {
		$host = self::fetch($host_id);
		$existing = array_key_exists('host_id',$host) && !empty($host['host_id']);		
		$callback_params = array ('host' => $host, 'existing' => $existing);
		add_meta_box('wpwhoosh-host-intro', __('Introduction',WPWHOOSH_PLUGIN_NAME), array(WPWHOOSH_HOSTS, 'intro_panel'), self::get_screen_id(), 'normal', 'core');
		add_meta_box('wpwhoosh-host-identity', __('Host Details',WPWHOOSH_PLUGIN_NAME), array(WPWHOOSH_HOSTS, 'id_panel'), self::get_screen_id(), 'normal', 'core', $callback_params);
		
	
		$current_screen = get_current_screen();
		if (method_exists($current_screen,'add_help_tab')) {		
			$current_screen->add_help_tab( array( 'id' => 'wpwhoosh_hosts_edit', 'title' => 'Overview', 		
			'content' => '<p>Here you can add and verify host information.</p>'));	
			$current_screen->add_help_tab( array( 'id' => 'wpwhoosh_hosts_support', 'title' => 'Troubleshooting', 		
			'content' => '<h3>Common Problems</h3><p>The most common problem is typing in the password incorrectly. Please use cut and paste for sake of reliability.</p>'));		
		}
	}

	static function controller() {
		WPWhooshUtils::check_permissions();
    	$message='';
    
		if ( isset($_REQUEST['message']) ) {
			$message = urldecode($_REQUEST['message']);
			$_SERVER['REQUEST_URI'] = remove_query_arg(array('message'), $_SERVER['REQUEST_URI']);
		}

		$action =  array_key_exists("action",$_REQUEST) ? $_REQUEST['action'] : -1;
		if (array_key_exists('id', $_GET)  || ($action=='add') || ($action=='save'))
			self::single_action($action, $message);
		else 
			self::bulk_actions($action, $message);
	}

	static function bulk_actions($action, $message) {
		if (isset( $_REQUEST['cb'] ) ) {
			check_admin_referer( 'bulk-hosts' );
			$ids =  (array) $_REQUEST['cb'];
			switch ($action) {
				case 'delete':  { $message = self::delete_rows($ids); break; }
				default: {}
			}
		} 
		WPWhooshUtils::clear_nonce();
		self::list_rows($message);
	}

	static function single_action($action, $message) {
		$id = array_key_exists('id', $_GET)  ? $_GET['id'] : 0;
		if ($host = self::fetch($id)) 	
			switch ($action) {		
				case 'add':
				case 'edit' : {	self::edit($host,$message); break; }
				case 'save' : { self::save(); break; }
				case 'update': { self::update($host); break; }			
				case 'delete' : { self::delete($host); break; }
				case 'copy' : { self::copy($host,$_POST['new_name']); break; }
				default: { 	wp_die(__('Unknown/invalid action for host.'));  } 
			}
		else
			wp_die ('Host '.$id.' not found');
	}

	static function fetch($id = 0) {
		if  (empty($id)) 
			return array_fill_keys(self::get_keys(),'');
		else 
			return WPWhooshUtils::get_host($id);
	}

	static function list_rows($message='') {
		$action_url = self::get_url();
		if (empty($message) && ! WPWhooshUpdater::check_validity()) $message = 'Validate your API key before adding a host';
		if (!empty($message)) $message = '<div id="message" class="updated"><p>'.$message.'</p></div>';
		print <<< HOSTS_LIST
<div class="wrap nosubsub">
<h2>Hosts <a href="{$action_url}&action=add" class="button add-new-h2">Add New</a></h2>
{$message}
<form id="posts-filter" action="{$action_url}" method="post" >
HOSTS_LIST;
		self::reset_vars();
	    self::get_list()->prepare_items();
	    self::get_list()->search_box( __( 'Search Hosts' ), 'host' ); 	    
		self::get_list()->display();
		echo('<div id="ajax-response"></div></form></div>');
	}

	static function delete_rows($ids) {
	    $deleted = 0;
		foreach ( $ids as $id ) { 
			if (($host=WPWhooshUtils::get_host($id)) && self::update_data( $host, true )) $deleted++; 
			}
		$message = sprintf(__('%d Hosts have been deleted.'), $deleted);	
	    return $message;
	}
	
	static function delete($host) {
    	$host_id = $host['host_id'];
		$redir = WPWhooshUtils::next_url('delete-host_' . $host_id); 
    	if (self::update_data( $host, true)) {
			$message = sprintf('Host %1$s was deleted successfully.',$host['host_name']); 
		} else {
			$message = sprintf('Host %1$s failed to be deleted.',$host['host_name']); 
			$redir = add_query_arg( array('host_id' => $host_id, 'action' => 'edit'), $redir );	//go back to edit page	
		}
		$redir = add_query_arg( array('message' => urlencode($message)), $redir ); //add the message 
    	wp_redirect( $redir ); 
    	exit;
	}

	static function save() {
	    $redir = WPWhooshUtils::next_url('add-host'); 
	    $host_name = WPWhooshUtils::clean_name($_POST['host_name']);
  		$host = self::sanitize_post();
		if (self::is_valid_host($host) ) {
			if (self::update_data($host)) {
				if (WPWhooshUpdater::check_validity())
    				if (self::login($host))
						$message = sprintf('Host %1$s was added and verified successfully.',$host_name);		
					else
						$message = sprintf('Host %1$s was added but login details could not be verified.',$host_name);
				else
					$message = sprintf('Host %1$s was added but cannot be verified until you register your API key.',$host_name);					
				$redir = add_query_arg( array('id' => $host['host_id'], 'action' => 'edit'), $redir );	//go to edit page			
			} else {
				$message = sprintf('Host %1$s failed to be added.',$host_name); 
				$redir = add_query_arg( array('action' => 'add'), $redir );	//go back to add page	
			}
		} else {
			$message = sprintf('Host information is incomplete. Hit the back key and enter the missing values.'); 
			$redir = add_query_arg(  array('action' => 'add'), $redir ); //go back to add page		}
		}
		$redir = add_query_arg( array('message' => urlencode($message)), $redir ); //add the message 
	    wp_redirect( $redir ); 
	    exit;
	}

	static function update($host) {
    	$host_id = $host['host_id'];
    	$redir = WPWhooshUtils::next_url('edit-host_' . $host_id); 
   		$host_name = WPWhooshUtils::clean_name($_POST['host_name']); //check for blank ID
		$host = self::sanitize_post();
  		if (self::is_valid_host($host) )
    		if (self::update_data($host)) 
    			if (WPWhooshUpdater::check_validity())
    				if (self::login($host))
						$message = sprintf('Host %1$s was updated and verified successfully.',$host_name);		
					else
						$message = sprintf('Host %1$s was updated but login details could not be verified.',$host_name);					
				else
					$message = sprintf('Host %1$s was updated but cannot be verified until you register your API key.',$host_name);					
			else 
				$message = sprintf('No changes were made to host %1$s.',$host_name);  
		else 
			$message = sprintf('All required fields must be supplied for %1$s. Go back and edit the host info.',$host_name); 
		$redir = add_query_arg( array('action' => 'edit', 'message' => urlencode($message)), $redir );  
    	wp_redirect( $redir ); 
    	exit;
	}

	static function generate_copy_host_name($original) {
		$suffix = 'copy';
		$num = 0;
		$root = $original.$suffix;
		if (strpos($original,$suffix) !== FALSE) 
			if (preg_match_all("|\d+|", $original, $m)) {
				$last = count($m) - 1;
				$num = $m[$last];
				$root = substr($original,0,strrpos($original,$num));
			}  				
		do {
			$num=+1;
			$new_host_name = $root.$num;
		} while (WPWhooshUtils::get_host_by_name($new_host_name));
		return $new_host_name;  //return an unique name
	}

	static function copy($host, $new_host_name='') {
		$host_id = $host['host_id'];
    	$redir = WPWhooshUtils::next_url('copy-host_' . $host_id); 
	    $new_host_name = WPWhooshUtils::clean_name($new_host_name); //clean_id
	    if (empty($new_host_name) || WPWhooshUtils::get_host_by_name($new_host_name)) 
	    	$new_host_name = self::generate_copy_host_name( $host['host_name']); //new name

		$new_host = array();
		$new_host['host_name'] = $new_host_name; //give it the new name		
		$new_host['cpanel_url'] = $host['cpanel_url'];
		$new_host['cpanel_user'] = $host['cpanel_user'];
		$new_host['cpanel_password'] = $host['cpanel_password'];
		if ($host_id = self::update_data($new_host))   //update and return new host ID
    		if (self::login($host))
				$message = sprintf('Host %1$s was copied and verified successfully.',$new_host_name);		
			else
				$message = sprintf('Host %1$s was copied but login details could not be verified.',$new_host_name);			
		else 
			$message = sprintf(__('Failed to create new host %1$s.'),$new_host_name);		
		$redir = add_query_arg( 
			array('id' => urlencode($host_id),'action' => 'edit','message' => urlencode($message)), $redir );  
    	wp_redirect( $redir ); 
    	exit;
 	}
	
	static function login(&$host) {
		$status = WPWhooshUpdater::update(false, array('action' => 'host_checks', 'timeout' => 30, 'body' => array( 'host' => $host)));
		if ($status['success']) {
		 	$host['cpanel_status'] = 'VERIFIED';
		 	$host['cpanel_verified_date'] = date('Y-m-d h:i');
			$verified = true;
		} else {
		 	$host['cpanel_status'] = 'INVALID';
		 	$host['cpanel_verified_date'] = '';
			$verified = false;
		}
		self::update_data($host);
		return $verified;
	}

	private static function is_valid_host($host) {
		if (!is_array($host)) $host = WPWhooshUtils::get_host($host);
		return is_array($host)
			&& array_key_exists('cpanel_url',$host) && ! empty($host['cpanel_url'])
			&& array_key_exists('cpanel_user',$host) && ! empty($host['cpanel_user'])
			&& array_key_exists('cpanel_password',$host) && ! empty($host['cpanel_password'])
			&& array_key_exists('host_name',$host) && ! empty($host['host_name']) ;
	}

	static function update_data( &$new_host, $delete=false) { 
		$old_host = array();
		if (! array_key_exists('host_id',$new_host) || empty($new_host['host_id'])) $new_host['host_id'] = WPWhooshUtils::generate_host_id(); 

		$host_id =  $new_host['host_id'] ;
		$all_hosts = WPWhooshUtils::get_hosts (false) ; //fetch from database - not the cache
		if (array_key_exists($host_id,$all_hosts)) {
			$old_host = $all_hosts[$host_id];
			unset($all_hosts[$host_id]); //delete the old host from the array
		}
	
		if (! $delete) {
			$revised_data = self::review_data($new_host,$old_host); //updated any derived fields / change of state
			$all_hosts[$host_id] = $revised_data ; //add the new or updated host
		}
		$new_host = $revised_data;
		if (WPWhooshUtils::save_hosts( $all_hosts))  //save to database
			return $host_id; //return current/deleted host id
		else 
			return false; //update false
 	}

	static function review_data($new_data,$old_data) {
		if (count($old_data)==0) return $new_data;
		
		if ( array_key_exists('cpanel_password',$old_data) && (md5($old_data['cpanel_password']) == $new_data['cpanel_password']))
			$new_data['cpanel_password'] = $old_data['cpanel_password'];

		if (($old_data['cpanel_status'] == 'VERIFIED')
		&& (($old_data['cpanel_url'] != $new_data['cpanel_url'])
		|| ($old_data['cpanel_user'] != $new_data['cpanel_user'])
		|| ($old_data['cpanel_password'] != $new_data['cpanel_password']))) {
		 		$new_data['cpanel_status'] = 'UNVERIFIED';
		 		$new_data['cpanel_verified_date'] = '';	
		}		
		return $new_data;
	}

	static function sanitize_post( ) {
		$_POST['host_name'] = WPWhooshUtils::clean_name($_POST['host_name']);
		$_POST['cpanel_url'] = WPWhooshUtils::clean_url($_POST['cpanel_url']);
		$_POST['cpanel_user'] = WPWhooshUtils::clean_name($_POST['cpanel_user']);		
		return $_POST;
	}

	static function reset_vars() {
		wp_reset_vars(array('action',  'submit',  'host_id', 'host_name','cpanel_url', 'cpanel_user', 'cpanel_password',
			'cpanel_status', 'cpanel_verified_date', 'cpanel_provider'));
	}

    static function toggle_postboxes() {
    	$slug = self::get_screen_id();
    	print <<< POSTBOXES
<script type="text/javascript">
//<![CDATA[
	jQuery(document).ready( function($) {
	// close postboxes that should be closed
		$('.if-js-closed').removeClass('if-js-closed').addClass('closed');
		// postboxes setup
		postboxes.add_postbox_toggles('{$slug}');
	});
//]]>
</script>
POSTBOXES;
    }

    private static function fetch_message($action) {
		$actions = empty($action) ? array() : (is_array($action) ? $action : array($action));
		$show_message = '' ;
		if (isset($_REQUEST['message']) && ! empty($_REQUEST['message'])) { 
			if ((count($actions) == 0)
			|| (array_key_exists('lastaction',$_REQUEST)  && in_array($_REQUEST['lastaction'],$actions))) {
				$message = urldecode($_REQUEST['message']);
				$style = strpos($message,'success') !== FALSE ? ' success' : (strpos($message,'fail') !== FALSE ? ' failure' : '');
				$show_message = sprintf ('<div id="message" class="%2$s"><div class="updated">%1$s</div></div>',$message,$style); 
				$_SERVER['REQUEST_URI'] = remove_query_arg(array('message'), $_SERVER['REQUEST_URI']);
			}
		}
		return $show_message;
    }  

	static function edit($host,$message='') {
		$existing = array_key_exists('host_id',$host) && !empty($host['host_id']);
		$host_id = $existing?esc_attr($host['host_id']):'';	
		$submit_text = __(($existing?'Update':'Add').' Host');
		echo('<div id="poststuff" class="metabox-holder has-right-sidebar">');
        echo ('<h2>'.($existing?'Edit':'Add').' Host</h2>');
		echo('<p>You need to fill in all the fields below.</p>');
		echo('<div id="side-info-column" class="inner-sidebar">');
		do_meta_boxes(self::get_screen_id(), 'side', null); 
        echo('</div>');
        echo('<div id="post-body" class="has-sidebar"><div id="post-body-content" >');
		echo('<form name="host_form" id="host_form" method="post" action="'.self::get_url($host_id, 'noheader').'">');
		echo('<input type="hidden" name="host_id" value="'.$host_id.'" />');
		echo('<input type="hidden" name="action" value="'.($existing?'update':'save').'" />');
		wp_nonce_field( $existing?('edit-host_' . $host_id):'add-host','_wpnonce',true);
		wp_nonce_field('closedpostboxes', 'closedpostboxesnonce', false );
		wp_nonce_field('meta-box-order', 'meta-box-order-nonce', false );
		do_meta_boxes(self::get_screen_id(), 'normal', null);
		echo('<p class="submit"><input type="submit" class="button-primary" name="save" id="save" accesskey="s" value="'.$submit_text.'" /></p>');
		echo('</form>');
		do_meta_boxes(self::get_screen_id(), 'advanced', null);
 		echo('</div></div><br class="clear"/></div>');
	}    
    	
	static function id_panel($post,$metabox){
		$host = $metabox['args']['host'];
		$existing = $metabox['args']['existing'];
		$host_id= $existing?esc_attr($host['host_id']):'';
		$host_name= $existing?esc_attr($host['host_name']):'';
		$tip1 = self::$tooltips->tip('host_name');
		$cpanel_provider = new wpwhoosh_provider_list();
		$cpanel_provider_options = $cpanel_provider->get_dropdown( $host['cpanel_provider']);
		$tip2 = self::$tooltips->tip('cpanel_provider');
		$cpanel_url= $existing?esc_attr($host['cpanel_url']):'';
		$tip3 = self::$tooltips->tip('cpanel_url');
		$cpanel_user= $existing?esc_attr($host['cpanel_user']):'';
		$tip4 = self::$tooltips->tip('cpanel_user');
		$cpanel_password= $existing?md5($host['cpanel_password']):'';
		$tip5 = self::$tooltips->tip('cpanel_password');
		$cpanel_status= $existing?ucwords(strtolower(esc_attr($host['cpanel_status']))):'';
		$cpanel_verified_date= $existing?esc_attr($host['cpanel_verified_date']):'';
		$tip6 = self::$tooltips->tip('cpanel_status');
		$message = self::fetch_message(array('save','update'));
		print <<< ID_PANEL
<h4>{$tip1}<input type="text" id="host_name" name="host_name" size="50" tabindex="2" value="{$host_name}" /></h4> 
<h4>{$tip2}{$cpanel_provider_options}</h4>
<h4>{$tip3}<input type="text" id="cpanel_url" name="cpanel_url" size="60" tabindex="3" value="{$cpanel_url}"  /></h4> 
<h4>{$tip4}<input type="text" id="cpanel_user" name="cpanel_user" size="10" tabindex="4" value="{$cpanel_user}" /></h4> 
<h4>{$tip5}<input type="password" name="cpanel_password" size="20" tabindex="5" value="{$cpanel_password}" id="cpanel_password" /></h4> 
<h4>{$tip6}{$cpanel_status} {$cpanel_verified_date} {$message}</h4>
ID_PANEL;
	}


	static function intro_panel($post){
		print <<< INTRO_PANEL
		<p>Fill in your host details. This is information is required so we can deploy your sites</p>
INTRO_PANEL;
	}
	
	static function copy_panel($post,$metabox) {
		$host = $metabox['args']['host'];
		$existing = $metabox['args']['existing'];
		$id = $host['host_id'];
		$nonce = wp_nonce_field('copy-host_'.$id,'_wpnonce',true,false);
		$action_url = self::get_url($id,'noheader');
		print <<< COPY_PANEL
<form method="POST" name="copyhost" action="{$action_url}">
{$nonce}
<input type="hidden" name="action" value="copy" />
<h4>New Host Name</h4> <input type="text" id="new_name" name="new_name" value="" />
<p>Specify the name of the new host which will become a copy of the host displayed on this page.</p>
<input type="submit" name="copy-host" value="Copy Host" />
</form>
COPY_PANEL;
	}

	static function links_panel($post,$metabox) {
		$host = $metabox['args']['host'];
		$existing = $metabox['args']['existing'];
	    $plugin_home = WPWHOOSH_SUPPORT;
		print <<< LINKS_PANEL
<ul>
<li><a href="{$plugin_home}11/">How To Add A Host</a></li>
<li><a href="{$plugin_home}13/">How To Verify A Host</a></li>
</ul>	
LINKS_PANEL;
	}

}

class wpwhoosh_provider_list extends wpwhoosh_list {
 	function __construct($list='cpanel_provider') { 
 		parent::__construct($list, WPWhooshUtils::get_list($list));
	}
}

?>