<?php
class wpwhoosh_key {

    private static $initialized = false;
    private static $slug = WPWHOOSH_KEY;
    private static $screen_id;
    private static $label;
    private static $pin;

	static function init() {
	    if (self::$initialized) return true;
		self::$initialized = true;
		self::$label = WPWHOOSH_FRIENDLY_NAME.' '.__('API Key',WPWHOOSH_PLUGIN_NAME);
		self::$pin = WPWHOOSH_FRIENDLY_NAME.' '.__('PIN',WPWHOOSH_PLUGIN_NAME);
		add_action('admin_menu', array(WPWHOOSH_KEY, 'admin_menu'));
	}

	static function admin_menu() {
		self::$screen_id = add_submenu_page(WPWHOOSH_PLUGIN_NAME, self::$label, __('API Key',WPWHOOSH_PLUGIN_NAME), 'read', 
			self::get_slug(), array(WPWHOOSH_KEY,'controller'));
		add_filter('screen_layout_columns', array(WPWHOOSH_KEY, 'screen_layout_columns'), 10, 2);
		add_action('load-'.self::get_screen_id(), array(WPWHOOSH_KEY, 'load_page'));
		add_action('admin_print_styles-'.self::get_screen_id(), array(WPWHOOSH_KEY, 'load_style'));
		add_action('admin_print_scripts-'.self::get_screen_id(), array(WPWHOOSH_KEY, 'enqueue_scripts')); //ok to enqueue scripts in the footer at this hook	
		add_action('admin_footer-'.self::get_screen_id(), array(WPWHOOSH_KEY, 'toggle_postboxes'));
	}

    static function get_slug() {
		return self::$slug;
	}

    static function get_screen_id(){
		return self::$screen_id;
	}

 	static function get_url($action='',$noheader=false,$with_nonce=false) {
 		return WPWhooshUtils::admin_url(self::get_slug(), $action, '', $noheader, $with_nonce);       
	}

	static function screen_layout_columns($columns, $screen) {
		if (!defined( 'WP_NETWORK_ADMIN' ) && !defined( 'WP_USER_ADMIN' )) {
			if ($screen == self::get_screen_id()) {
				$columns[self::get_screen_id()] = 2;
			}
		}
		return $columns;
	}

	static function load_style() {
    	printf ('<link rel="stylesheet" id="%1$s" href="%2$s?ver=%3$s" type="text/css" media="all" />',
			WPWHOOSH_KEY, WPWHOOSH_PLUGIN_URL.'key.css', WPWHOOSH_VERSION);
 	}

	public static function enqueue_scripts() {
		wp_enqueue_script('key', WPWHOOSH_PLUGIN_URL.'key.js', array('jquery'), WPWHOOSH_VERSION, true);
		wp_enqueue_script('crypto',WPWHOOSH_PLUGIN_URL.'md5.js',array(),'3.0.2',true);			
	}


	static function load_page() {
		wp_enqueue_script('common');
		wp_enqueue_script('wp-lists');
		wp_enqueue_script('postbox');	
		add_meta_box('api-key-panel', self::$label, array(WPWHOOSH_KEY, 'key_panel'), self::get_screen_id(), 'normal', 'core');
		add_meta_box('pin-panel', self::$pin, array(WPWHOOSH_KEY, 'pin_panel'), self::get_screen_id(), 'normal', 'core');
		add_meta_box('request-panel', __('Request API Key',WPWHOOSH_PLUGIN_NAME), array(WPWHOOSH_KEY, 'request_panel'), self::get_screen_id(), 'side', 'core');
		$current_screen = get_current_screen();
		if (method_exists($current_screen,'add_help_tab')) {		
			$current_screen->add_help_tab( array( 'id' => 'wpwhoosh_key_overview', 'title' => self::$label, 		
			'content' => sprintf('<h4>Get Your %1$s</h4><p>%2$s. %3$s</p><p><a rel="external" href="%4$s">%5$s %1$s</a></p>',
				self::$label, 
				__('Here you can fetch your API Key by completing the form below',WPWHOOSH_PLUGIN_NAME),
				__('For more detailed instructions check out the tutorials',WPWHOOSH_PLUGIN_NAME),
				WPWHOOSH_HOME_URL.'/tutorials/',
				__('Getting Started with',WPWHOOSH_PLUGIN_NAME)
			)));		
			$current_screen->add_help_tab( array( 'id' => 'wpwhoosh_pin_overview', 'title' => self::$pin, 		
			'content' => self::pin_help_panel()));		
		}	
	}

    static function toggle_postboxes() {
    	$hook = self::get_screen_id();
    	print <<< TOGGLE_POSTBOXES
<script type="text/javascript">
//<![CDATA[
		jQuery(document).ready( function($) {
			$('.if-js-closed').removeClass('if-js-closed').addClass('closed');
			postboxes.add_postbox_toggles('{$hook}');
		});
//]]>
</script>
TOGGLE_POSTBOXES;
    }

	static function save() {
		$redir = WPWhooshUtils::next_url('save');
		$key = trim(stripslashes($_POST['key']));
		$message = self::$label.' '.__(WPWhooshUpdater::save_key($key) ? "saved" : "has not changed" ,WPWHOOSH_PLUGIN_NAME);
 		WPWhooshUtils::save_templates(WPWhooshUpdater::update(false)); //update with new entitlements as a licensed user
  		$redir = add_query_arg( array('message' => urlencode($message)), $redir );  
    	wp_redirect( $redir ); 
    	exit;
	}

	private static function change_pin() {
    	$redir = WPWhooshUtils::next_url('change_pin'); //check nonce 
		$args = array('action' => 'pin_change', 'timeout' => 30, 
			'body' => array('pin' => $_POST['pin'], 'auth' => $_POST['authorization']));
		$status = WPWhooshUpdater::update(false, $args);
		if ($status['success']) {
			$message = __('Your PIN has been changed',WPWHOOSH_PLUGIN_NAME);
		} else {
			$message = __('PIN changes are not available at this time:',WPWHOOSH_PLUGIN_NAME). $status['error'];
		}
		$redir = add_query_arg( array('message' => urlencode($message)), $redir );  
    	wp_redirect( $redir ); 
    	exit;
	}

	private static function reset_pin() {
    	$redir = WPWhooshUtils::next_url('reset_pin'); 
		$args = array('action' => 'pin_reset', 'timeout' => 30);
		$status = WPWhooshUpdater::update(false, $args);
		if ($status['success']) {
			$message = __('Check your email for the authorization code you will need to change your PIN',WPWHOOSH_PLUGIN_NAME);
		} else {
			$message = __('PIN changes are not available at this time:',WPWHOOSH_PLUGIN_NAME). $status['error'];
		}
		$redir = add_query_arg( array('message' => urlencode($message)), $redir );  
    	wp_redirect( $redir ); 
    	exit;
	}

	static function key_panel($post, $metabox) {		
		$home = WPWHOOSH_HOME_URL;	
		$friendly_name = WPWHOOSH_FRIENDLY_NAME;	
		$is_valid = false;
		$key_status_indicator ='';
		$notice ='';
		$key = WPWhooshUpdater::get_key(false);
		if (! empty($key)) {
   			$is_valid = WPWhooshUpdater::check_validity();
   			$flag = $is_valid ? 'tick' : 'cross';
   			$key_status_indicator = '<img src="' . WPWHOOSH_PLUGIN_URL .'/images/'.$flag.'.png" alt="a '.$flag.'" />';
 			$notice = WPWhooshUpdater::get_notice();
 		}
        $readonly = $is_valid ? '' : 'readonly="readonly" class="readonly"';
 		$this_url = self::get_url('save',true,true); 		        
		$closed_nonce = wp_nonce_field('closedpostboxes', 'closedpostboxesnonce', false, false); 
		$order_nonce = wp_nonce_field('meta-box-order', 'meta-box-order-nonce', false, false);
		print <<< LICENCE_PANEL
<h4>How To Get A {$friendly_name} API Key</h4>
<p>To get an API key: </p>
<ol>
<li>Fill out the form on the right</li>
<li>The first email we'll send will contain a link which you need to click to confirm your email address</li>
<li>Once you've done that we'll send a second email containing your API key</li>
<li>Copy and paste the API key into the box below and click on the <i>Save Changes</i></li>
<li>You're done!</li>
<li>Get back to us with any problems by visiting <a href="{$home}/help/">our support page</a></li>
</ol>
<form id="api_key_entry" method="post" action="{$this_url}"> 
<label for="key">API Key: </label><input type="password" name="key" id="key"  style="width:320px" value="{$key}" />&nbsp;{$key_status_indicator}
<p class="api-key-notice">{$notice}</p>
<p><input type="submit" class="button-primary" name="options_update" value="Save Changes" /></p>
<p class="submit"><input type="hidden" name="action" value="save" />{$closed_nonce}{$order_nonce}</p>
</form>
LICENCE_PANEL;
	}

	static function pin_panel($post, $metabox) {		
		$home = WPWHOOSH_HOME_URL;	
		$pin = self::$pin;	
		$is_valid = false;
		$notice ='';
 		$reset_url = self::get_url('reset_pin',true,true);       
 		$change_url = self::get_url('change_pin',true,true);       
		print <<< PIN_PANEL
<h4>How Change Your {$pin}</h4>
<p>There are are two steps:</p>
<p>1. Request a PIN Reset: Click <a href="{$reset_url}">Reset PIN</a></p>
<p>2. Check your email and copy the authorization code and paste it below next to your new PIN</p>
<form id="pin_entry" method="post" onsubmit="return wpwhoosh_validate_pin(this);" action="{$change_url}"> 
<label for="pin">PIN: </label><input type="password" name="pin" id="pin"  style="width:100px" maxlength="8" />
<label for="authorization">Authorization: </label><input type="text" name="authorization" id="authorization"  style="width:320px"  />
<p><input type="hidden" name="action" value="change_pin"/><input type="submit" class="button-primary" value="Change PIN" /></p>
</form>
PIN_PANEL;
	}

	static function pin_help_panel() {		
		$home = WPWHOOSH_HOME_URL;	
		$friendly_name = WPWHOOSH_FRIENDLY_NAME;	
		$label = self::$label;
		$pin = self::$pin;
		$panel = <<< PIN_HELP_PANEL
<h4>Protect Your {$label} With A {$pin}</h4>
<p>With {$friendly_name} you may elect not to use a PIN. However, if you are spending a significant
number of credits each month you may want to protect your API key with a PIN (personal identification number). This can be between 4 and 8 digits long. Every time you spend credits then you 
will be prompted to enter your PIN. The PIN is encrypted in the browser and then sent as an authorization of the installation request.</p> 
<p>To maintain security, your {$pin} is never displayed or sent as free text.</p>
<p>If you lose or forget your PIN, or simply want to change it, you can do so by following a two-step procedure. Firstly you request
a change of PIN below and the system will email your primary email address an authorization code which lasts for 15 minutes. Secondly, you then 
choose a new PIN and enter the authorization code. If the authorization code is valid the new PIN will become active.</p>
<p>If you set an empty new PIN, then the PIN challenge will be disabled.</p>
PIN_HELP_PANEL;
		return $panel;
	}

    private static function fetch_message($action) {
		$actions = empty($action) ? array() : (is_array($action) ? $action : array($action));
		$show_message = '' ;
		if (isset($_REQUEST['message']) && ! empty($_REQUEST['message'])) { 
			if ((count($actions) == 0)
			|| (array_key_exists('lastaction',$_REQUEST)  && in_array($_REQUEST['lastaction'],$actions))) {
				$message = urldecode($_REQUEST['message']);
				$style = strpos($message,'success') !== FALSE ? ' success' : (strpos($message,'fail') !== FALSE ? ' failure' : '');
				$show_message = sprintf ('<div id="message" class="%2$s"><div class="updated">%1$s</div></div>',$message,$style); 
				$_SERVER['REQUEST_URI'] = remove_query_arg(array('message'), $_SERVER['REQUEST_URI']);
			}
		}
		return $show_message;
    }  

	static function request_panel($post, $metabox) {
		$plugin = WPWHOOSH_PLUGIN_NAME;
		$home = WPWHOOSH_HOME_URL;
		$images = WPWHOOSH_IMAGES_URL;
		$domain = parse_url(site_url(),PHP_URL_HOST);
		print <<< REQUEST_PANEL
<img src="{$images}get-api-key.png" alt="API Key Request" />
<form id="api_key_signup" name="api_key_signup" method="post" action="{$home}/" onsubmit="return wpwhoosh_validate_form(this);">
<input type="hidden" name="form_storm" value="submit" />
<input type="hidden" name="destination" value="{$plugin}" />
<input type="hidden" name="domain" value="{$domain}" />
<label for="firstname">First Name</label><br/><input id="firstname" name="firstname" type="text" value="" /><br/>
<label for="email">Your Email</label><br/><input id="email" name="email" type="text" /><br/>
<label id="lsubject" for="subject">Subject<input id="subject" name="subject" type="text" /></label>
<input type="submit" value="" />
</form>
REQUEST_PANEL;
	}	

	static function controller() {
 		switch ($_REQUEST['action']) {
 		 case 'reset_pin' : self::reset_pin(); break;
 		 case 'change_pin': self::change_pin(); break;
 		 case 'save': self::save(); break;
 		}
 		$this_url = $_SERVER['REQUEST_URI']; 		
    	$templates_url = WPWHOOSH_TEMPLATES_URL; 
		echo self::fetch_message(array('reset_pin','change_pin','save'));
?>
    <div id="poststuff" class="metabox-holder has-right-sidebar">
        <h2>Fetching your API Key</h2>
		<p>You need an API key for you to be able to connect with our server and complete a remote WordPress installation.</p>
		<p>Through our membership site you can get access to <a href="<?php echo $templates_url; ?>">MANY MORE templates</a>.</p>
        <div id="side-info-column" class="inner-sidebar">
		<?php do_meta_boxes(self::get_screen_id(), 'side', null); ?>
        </div>
        <div id="post-body" class="has-sidebar">
            <div id="post-body-content" class="has-sidebar-content">
			<?php do_meta_boxes(self::get_screen_id(), 'normal', null); ?>
 			</div>
        </div>
        <br class="clear"/>
    </div>
<?php
	}  

}
?>