<?php
define ('WPWHOOSH_PUBLIC','wpwhoosh_public');
add_action ('init', array(WPWHOOSH_PUBLIC, 'init' ) );

class wpwhoosh_public {

	const REMOTE_KEY = 'wpwhoosh_remote';

	static function init() {
		remove_action( 'wp_head', 'wp_generator' ); //don't show WordPress version
		remove_action( 'wp_head', 'wlwmanifest_link' ); //don't show Manifest Link
		if (self::allow_remote()) {
			add_filter ('xmlrpc_methods', array(WPWHOOSH_PUBLIC, 'xmlrpc_methods' ) );
			add_filter ('xmlrpc_blog_options',array(WPWHOOSH_PUBLIC, 'xmlrpc_blog_options' ) );
		}
	}

	static function xmlrpc_methods($methods) {
	    	$methods[WPWHOOSH_PLUGIN_NAME . '.remoteMethod'] = array(WPWHOOSH_PUBLIC, 'xmlrpc_handler') ;
	    	return $methods;	
	}

	static function xmlrpc_handler($params) {
    	$username = $params[0];
    	$password = $params[1];
    	$method   = $params[2];
    	$args     = $params[3];

    	global $wp_xmlrpc_server;
    	if ( !$user = $wp_xmlrpc_server->login($username, $password) ) {
            return $wp_xmlrpc_server->error;
    	}    

    	if (self::xmlrpc_is_permitted_method($method)) {
        	try {
        	    return call_user_func_array($method, array($args));
        	} catch (Exception $e) {
        	    return new IXR_Error( 401, __( 'Remote call failed for '.$method . ' failed.',WPWHOOSH_PUBLIC) );
        	}
    	} else {
			return new IXR_Error( 401, __( 'Remote call for ' . $method . ' is not available.',WPWHOOSH_PUBLIC ) );
    	}	
	}
	
	static function xmlrpc_blog_options($options) {
		return $options + array(
			'admin_email' => array('desc' => __( 'Administration Email',WPWHOOSH_PUBLIC),'readonly' => false,'option'	=> 'admin_email'),
			'cron_backup_recipient'	=> array( 'desc' => __( 'Email address for scheduled database backups',WPWHOOSH_PUBLIC ), 'readonly' => false, 'option' => 'cron_backup_recipient'),
			'wpdb_backup_recip'	=> array( 'desc' => __( 'Email address for database backups',WPWHOOSH_PUBLIC ), 'readonly' => false, 'option' => 'wpdb_backup_recip'),
			'permalink_structure' => array( 'desc' => __( 'Permalink Structure',WPWHOOSH_PUBLIC),'readonly' => false, 'option' => 'permalink_structure'),
			'blog_public' => array( 'desc' => __( 'Control if Blog is Open To Search Engines',WPWHOOSH_PUBLIC ), 'readonly' => false, 'option' => 'blog_public'),
			'upload_path' => array('desc' => __( 'Media Library Folder',WPWHOOSH_PUBLIC ), 'readonly' => false, 'option' => 'upload_path'),
			'jetpack_activated' => array('desc' => __( 'JetPack activation status',WPWHOOSH_PUBLIC ), 'readonly' => false, 'option' => 'jetpack_activated'),
			'wordpress_api_key'	=> array('desc' => __( 'WordPress API Key Used for Stats collection',WPWHOOSH_PUBLIC ), 'readonly' => false, 'option' => 'wordpress_api_key')
			);
	}

	static function xmlrpc_is_permitted_method($method) {
		$permitted_functions = array(
			WPWHOOSH_PUBLIC.'::update_footer_credits', 
			WPWHOOSH_PUBLIC.'::update_user', 
			WPWHOOSH_PUBLIC.'::update_widget_twitter_name', 
			WPWHOOSH_PUBLIC.'::update_widget_flickr_id', 
			WPWHOOSH_PUBLIC.'::update_widget_facebook_url',
			WPWHOOSH_PUBLIC.'::update_contact_recipient',
			WPWHOOSH_PUBLIC.'::update_header_image',
			WPWHOOSH_PUBLIC.'::disable_jetpack',
			WPWHOOSH_PUBLIC.'::disable_remote');
    	return is_callable($method) && in_array($method, $permitted_functions);
	}

	static function disable_remote() {
		return update_option(self::REMOTE_KEY, false); 
	}

	static function allow_remote() {
		return get_option(self::REMOTE_KEY); 
	}
	
	static function update_footer_credits($footer_credits) {
		$updater = defined('FOOTER_CREDITS') ? array(FOOTER_CREDITS,'save'): false;
		if ($updater && is_callable($updater))
			return call_user_func($updater, $footer_credits);
		else 
			return false;
		
	}

	static function update_user($user) {
		if($userdata = get_user_by('login',$user['user_login'])) {
			$user['ID'] = $userdata->ID;
			return wp_update_user($user);
		} else {
			return false;
		}
	}	

	/* Until WordPress provide an API to update a widget, digging around the database will have to suffice */	
	static function update_widget_twitter_name($args) {
		$ukey = 'twitter_name';
		if ($args && is_array($args) && array_key_exists($ukey,$args) && !empty($args[$ukey])) {
			$widget_key = 'widget_twitter';
			$widgets = get_option($widget_key);		
			if ($widgets && is_array($widgets))		
			  foreach ($widgets as $key => $widget)
				if (is_array($widget) && array_key_exists('username',$widget)) {
					$widgets[$key]['username'] = $args[$ukey];
					update_option('widget_twitter',$widgets);
					return true;
				}
		}	
		return false;
	}
	
	static function update_widget_facebook_url($args) {
		$ukey = 'facebook_url';
		if ($args && is_array($args) && array_key_exists($ukey,$args) && !empty($args[$ukey])) {
			$widget_key = 'widget_facebook-likebox';
			$widgets = get_option($widget_key);			
			if ($widgets && is_array($widgets))
			  foreach ($widgets as $key => $widget)
				if (is_array($widget) && array_key_exists('like_args',$widget)
				&& is_array($widget['like_args']) && array_key_exists('href',$widget['like_args']) ) {
					$widgets[$key]['like_args']['href'] = $args[$ukey];
					update_option('widget_facebook-likebox',$widgets);
					return true;
				}
		}	
		return false;
	}

	static function update_widget_flickr_id($args) {
		$ukey = 'flickr_id';
		if ($args && is_array($args) && array_key_exists($ukey,$args) && !empty($args[$ukey])) {
			$option_key = 'slickr_flickr_options';
			$widget_key = 'widget_text';
			$widgets = get_option($widget_key);
			if ($widgets && is_array($widgets))
				foreach ($widgets as $key => $widget)
					if (is_array($widget) && array_key_exists('text',$widget)
					&& (strpos($widget['text'],'[slickr-flickr') !== false) ) {
						$options = get_option($option_key);
						$options[$ukey] = $args[$ukey];
						update_option($option_key,$options);
						return true;
					}
		}	
		return false;
	}
	
	static function update_contact_recipient($args) {
		$ukey = 'recipient';
		if ($args 
		&& is_array($args) 
		&& array_key_exists('old_url',$args)
		&& array_key_exists('new_url',$args)
		&& array_key_exists($ukey,$args) && !empty($args[$ukey])
		&& ($form = get_page_by_path('contact-form-1',OBJECT,'wpcf7_contact_form')) && !is_null($form)
		&& ($mail_config = get_post_meta($form->ID,'_mail',true))) {
				$mail_config[$ukey] = $args[$ukey];
				$mail_config['body'] = str_replace($args['old_url'],$args['new_url'], $mail_config['body']);
				return update_post_meta($page->ID,'_mail',$mail_config);
		}
		return false;
	}
	
	static function update_header_image($args) {
		if ($args 
		&& is_array($args) 
		&& array_key_exists('old_url',$args)
		&& array_key_exists('new_url',$args)
		&& ($mods = get_theme_mods())
		&& (strpos(serialize($mods), $args['old_url']) !== FALSE)) {
				if ($header_image = get_theme_mod('header_image')) {
					$header_image = str_replace($args['old_url'],$args['new_url'], $header_image);
					set_theme_mod('header_image', $header_image);
				}
				if ($header_image_data = get_theme_mod('header_image_data')) {
					$header_image_data->url = str_replace($args['old_url'],$args['new_url'], $header_image_data->url);
					$header_image_data->thumbnail_url = str_replace($args['old_url'],$args['new_url'], $header_image_data->thumbnail_url);
					set_theme_mod('header_image_data', $header_image_data);
				}
		}
		return false;
	}	
	
	static function disable_jetpack() {
		$options = get_option('jetpack_options');
		if ($options && array_key_exists('user_token',$options)) unset ($options['user_token']); 
		if ($options && array_key_exists('blog_token',$options)) unset ($options['blog_token']); 
		if ($options && array_key_exists('time_diff',$options)) unset ($options['time_diff']); 
		update_option('jetpack_options',$options);
		if(get_option('jetpack_activated')) update_option('jetpack_activated',false);
	}	
	
}
?>